/**************************************************************************************
 Copyright (c) 2002 The UbixOS Project
 All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions, the following disclaimer and the list of authors.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions, the following disclaimer and the list of authors
in the documentation and/or other materials provIded with the distribution. Neither the name of the UbixOS Project nor the names of its
contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 $Id: schedule.h,v 2.15 2003/05/11 23:37:07 reddawg Exp $

**************************************************************************************/

#ifndef _SCHEDULE_H
#define _SCHEDULE_H

#include <ubixos/types.h>
#include <ubixos/mount.h>
#include <ubixos/elf.h>
#include <sys/tss.h>

#ifdef __cplusplus
extern "C" {
#endif

extern int numTasks;

// Possible states for tasks to be in.
enum {
    STARTING = 0, READY, RUNNING, SUSPEND, SLEEP, BLOCK, 
    MSGREPLY, MSGWAIT, JOIN, EXITING, EMPTY
};

struct osInfo {
  struct mountPoints *container;
  elfSectionheader   *sectionHeader;
  uInt16 sectionCount;
  uInt16 stringSection;
  uInt8  timer;
  uInt8  v86Task;
  bool   v86If;
  uInt32 curDir;
  uInt32 vmStart;
  uInt32 stdinSize;
  uInt32 controlKeys;
  char   *stdin;
  char   *shstrtab;
  };

typedef struct taskStruct {
  struct tssStruct  tss;
  struct i387Struct i387;
  Int32 id;
  uInt16 status;
  uInt32 sleepTimeSec;
  uInt32 sleepTimeMs;
  uInt16 usedMath;
  uInt32 cpuTime;
  uInt32 lastExecutedTime;
  Int16  uid;
  Int16  gid;
  struct osInfo    oInfo;
  } kTask_t;

//extern kTask_t *taskList;
extern kTask_t *_current,*_usedMath;
extern int currentProc;

void kpanic(char *);

void initScheduler();
kTask_t * findTask();
kTask_t * getTask(int taskId);
void setTaskStatus(int taskId, uInt16 status);
void setTaskStatus_task(kTask_t * task, uInt16 status);
void timerInt();
void schedule();
void sched_yield();
void sleep(uInt32 sec, uInt32 mSec); // sleeps current task.
void endTask(pidType);

extern const uInt32 schedMagicNumber;

struct waitQueueNode {
  int id;
  void * data;
  struct waitQueueNode * next;
  struct waitQueueNode * prev;
  };

typedef struct waitQueue {
  uInt32 magic_number;
  uInt32 type;
  struct waitQueueNode * first;
  struct waitQueueNode * last;
  } waitQueue_t;

/*
waitQueue_t * waitQueueCreate(int);
void waitQueueInit(waitQueue_t *, int);
void waitQueueDelete(waitQueue_t *);
void waitQueueUnInit(waitQueue_t *);

void waitQueueInsert(waitQueue_t *, void *, int);
void * waitQueueRemove(waitQueue_t *, int *);
void * waitQueueFind(waitQueue_t *, int);
*/

#ifdef __cplusplus
}
#endif

#endif
